@@ -23,6 +23,8 @@
 #include "rules.h"
 #include "settings.h"
 #include "utils.h"
+#include "draw.h"
+#include "icon-lookup.h"
 
 static void notification_extract_urls(struct notification *n);
 static void notification_format_message(struct notification *n);
@@ -64,8 +66,10 @@ void notification_print(const struct notification *n)
         printf("\tformatted: '%s'\n", n->msg);
         printf("\tfg: %s\n", n->colors.fg);
         printf("\tbg: %s\n", n->colors.bg);
+        printf("\thighlight: %s\n", n->colors.highlight);
         printf("\tframe: %s\n", n->colors.frame);
         printf("\tfullscreen: %s\n", enum_to_string_fullscreen(n->fullscreen));
+        printf("\tformat: %s\n", n->format);
         printf("\tprogress: %d\n", n->progress);
         printf("\tstack_tag: %s\n", (n->stack_tag ? n->stack_tag : ""));
         printf("\tid: %d\n", n->id);
@@ -97,6 +101,7 @@ void notification_print(const struct notification *n)
                 printf("\n");
         }
         printf("}\n");
+        fflush(stdout);
 }
 
 /* see notification.h */
@@ -127,11 +132,32 @@ void notification_run_script(struct notification *n)
                         int status;
                         waitpid(pid1, &status, 0);
                 } else {
+                        // second fork to prevent zombie processes
                         int pid2 = fork();
                         if (pid2) {
                                 exit(0);
                         } else {
-                                int ret = execlp(script,
+                                // Set environment variables
+                                gchar *n_id_str = g_strdup_printf("%i", n->id);
+                                gchar *n_progress_str = g_strdup_printf("%i", n->progress);
+                                gchar *n_timeout_str = g_strdup_printf("%li", n->timeout/1000);
+                                gchar *n_timestamp_str = g_strdup_printf("%li", n->timestamp / 1000);
+                                safe_setenv("DUNST_APP_NAME",  appname);
+                                safe_setenv("DUNST_SUMMARY",   summary);
+                                safe_setenv("DUNST_BODY",      body);
+                                safe_setenv("DUNST_ICON_PATH", n->icon_path);
+                                safe_setenv("DUNST_URGENCY",   urgency);
+                                safe_setenv("DUNST_ID",        n_id_str);
+                                safe_setenv("DUNST_PROGRESS",  n_progress_str);
+                                safe_setenv("DUNST_CATEGORY",  n->category);
+                                safe_setenv("DUNST_STACK_TAG", n->stack_tag);
+                                safe_setenv("DUNST_URLS",      n->urls);
+                                safe_setenv("DUNST_TIMEOUT",   n_timeout_str);
+                                safe_setenv("DUNST_TIMESTAMP", n_timestamp_str);
+                                safe_setenv("DUNST_STACK_TAG", n->stack_tag);
+                                safe_setenv("DUNST_DESKTOP_ENTRY", n->desktop_entry);
+
+                                execlp(script,
                                                 script,
                                                 appname,
                                                 summary,
@@ -139,10 +165,9 @@ void notification_run_script(struct notification *n)
                                                 icon,
                                                 urgency,
                                                 (char *)NULL);
-                                if (ret != 0) {
-                                        LOG_W("Unable to run script: %s", strerror(errno));
-                                        exit(EXIT_FAILURE);
-                                }
+
+                                LOG_W("Unable to run script %s: %s", n->scripts[i], strerror(errno));
+                                exit(EXIT_FAILURE);
                         }
                 }
         }
@@ -170,7 +195,7 @@ const char *notification_urgency_to_string(const enum urgency urgency)
 /* see notification.h */
 int notification_cmp(const struct notification *a, const struct notification *b)
 {
-        if (a->urgency != b->urgency) {
+        if (settings.sort && a->urgency != b->urgency) {
                 return b->urgency - a->urgency;
         } else {
                 return a->id - b->id;
@@ -183,8 +208,6 @@ int notification_cmp_data(const void *va, const void *vb, void *data)
         struct notification *a = (struct notification *) va;
         struct notification *b = (struct notification *) vb;
 
-        ASSERT_OR_RET(settings.sort, 1);
-
         return notification_cmp(a, b);
 }
 
@@ -193,10 +216,34 @@ bool notification_is_duplicate(const struct notification *a, const struct notifi
         return STR_EQ(a->appname, b->appname)
             && STR_EQ(a->summary, b->summary)
             && STR_EQ(a->body, b->body)
-            && (settings.icon_position != ICON_OFF ? STR_EQ(a->icon_id, b->icon_id) : 1)
+            && (a->icon_position != ICON_OFF ? STR_EQ(a->icon_id, b->icon_id) : 1)
             && a->urgency == b->urgency;
 }
 
+bool notification_is_locked(struct notification *n) {
+        assert(n);
+
+        return g_atomic_int_get(&n->locked) != 0;
+}
+
+struct notification* notification_lock(struct notification *n) {
+        assert(n);
+
+        g_atomic_int_set(&n->locked, 1);
+        notification_ref(n);
+
+        return n;
+}
+
+struct notification* notification_unlock(struct notification *n) {
+        assert(n);
+
+        g_atomic_int_set(&n->locked, 0);
+        notification_unref(n);
+
+        return n;
+}
+
 static void notification_private_free(NotificationPrivate *p)
 {
         g_free(p);
@@ -229,6 +276,8 @@ void notification_unref(struct notification *n)
         g_free(n->summary);
         g_free(n->body);
         g_free(n->iconname);
+        g_free(n->default_icon_name);
+        g_free(n->icon_path);
         g_free(n->msg);
         g_free(n->dbus_client);
         g_free(n->category);
@@ -236,38 +285,70 @@ void notification_unref(struct notification *n)
         g_free(n->urls);
         g_free(n->colors.fg);
         g_free(n->colors.bg);
+        g_free(n->colors.highlight);
         g_free(n->colors.frame);
         g_free(n->stack_tag);
         g_free(n->desktop_entry);
 
         g_hash_table_unref(n->actions);
+        g_free(n->default_action_name);
 
         if (n->icon)
-                g_object_unref(n->icon);
+                cairo_surface_destroy(n->icon);
         g_free(n->icon_id);
 
         notification_private_free(n->priv);
 
-        if (n->script_count > 0){
+        if (n->script_count > 0) {
                 g_free(n->scripts);
         }
 
         g_free(n);
 }
 
+void notification_transfer_icon(struct notification *from, struct notification *to)
+{
+        if (from->iconname && to->iconname
+                        && strcmp(from->iconname, to->iconname) == 0){
+                // Icons are the same. Transfer icon surface
+                to->icon = from->icon;
+
+                // prevent the surface being freed by the old notification
+                from->icon = NULL;
+        }
+}
+
 void notification_icon_replace_path(struct notification *n, const char *new_icon)
 {
         ASSERT_OR_RET(n,);
         ASSERT_OR_RET(new_icon,);
-        ASSERT_OR_RET(n->iconname != new_icon,);
+        if(n->iconname && n->icon && strcmp(n->iconname, new_icon) == 0) {
+                return;
+        }
 
-        g_free(n->iconname);
-        n->iconname = g_strdup(new_icon);
+        // make sure it works, even if n->iconname is passed as new_icon
+        if (n->iconname != new_icon) {
+                g_free(n->iconname);
+                n->iconname = g_strdup(new_icon);
+        }
 
-        g_clear_object(&n->icon);
+        cairo_surface_destroy(n->icon);
+        n->icon = NULL;
         g_clear_pointer(&n->icon_id, g_free);
 
-        n->icon = icon_get_for_name(new_icon, &n->icon_id);
+        g_free(n->icon_path);
+        n->icon_path = get_path_from_icon_name(new_icon, n->min_icon_size);
+        if (n->icon_path) {
+                GdkPixbuf *pixbuf = get_pixbuf_from_file(n->icon_path,
+                                n->min_icon_size, n->max_icon_size,
+                                draw_get_scale());
+                if (pixbuf) {
+                        n->icon = gdk_pixbuf_to_cairo_surface(pixbuf);
+                        g_object_unref(pixbuf);
+                } else {
+                        LOG_W("No icon found in path: '%s'", n->icon_path);
+                }
+        }
 }
 
 void notification_icon_replace_data(struct notification *n, GVariant *new_icon)
@@ -275,10 +356,15 @@ void notification_icon_replace_data(struct notification *n, GVariant *new_icon)
         ASSERT_OR_RET(n,);
         ASSERT_OR_RET(new_icon,);
 
-        g_clear_object(&n->icon);
+        cairo_surface_destroy(n->icon);
+        n->icon = NULL;
         g_clear_pointer(&n->icon_id, g_free);
 
-        n->icon = icon_get_for_data(new_icon, &n->icon_id);
+        GdkPixbuf *icon = icon_get_for_data(new_icon, &n->icon_id,
+                        draw_get_scale(), n->min_icon_size, n->max_icon_size);
+        n->icon = gdk_pixbuf_to_cairo_surface(icon);
+        if (icon)
+                g_object_unref(icon);
 }
 
 /* see notification.h */
@@ -322,16 +408,26 @@ struct notification *notification_create(void)
 
         /* Unparameterized default values */
         n->first_render = true;
-        n->markup = settings.markup;
+        n->markup = MARKUP_FULL;
         n->format = settings.format;
 
         n->timestamp = time_monotonic_now();
 
         n->urgency = URG_NORM;
         n->timeout = -1;
+        n->dbus_timeout = -1;
 
         n->transient = false;
         n->progress = -1;
+        n->word_wrap = true;
+        n->ellipsize = PANGO_ELLIPSIZE_MIDDLE;
+        n->alignment = PANGO_ALIGN_LEFT;
+        n->progress_bar_alignment = PANGO_ALIGN_CENTER;
+        n->hide_text = false;
+        n->icon_position = ICON_LEFT;
+        n->min_icon_size = 32;
+        n->max_icon_size = 32;
+        n->receiving_raw_icon = false;
 
         n->script_run = false;
         n->dbus_valid = false;
@@ -339,6 +435,7 @@ struct notification *notification_create(void)
         n->fullscreen = FS_SHOW;
 
         n->actions = g_hash_table_new_full(g_str_hash, g_str_equal, g_free, g_free);
+        n->default_action_name = g_strdup("default");
 
         n->script_count = 0;
         return n;
@@ -363,17 +460,6 @@ void notification_init(struct notification *n)
         if (n->timeout < 0)
                 n->timeout = settings.timeouts[n->urgency];
 
-        /* Icon handling */
-        if (STR_EMPTY(n->iconname))
-                g_clear_pointer(&n->iconname, g_free);
-        if (!n->icon && n->iconname) {
-                char *icon = g_strdup(n->iconname);
-                notification_icon_replace_path(n, icon);
-                g_free(icon);
-        }
-        if (!n->icon && !n->iconname)
-                notification_icon_replace_path(n, settings.icons[n->urgency]);
-
         /* Color hints */
         struct notification_colors defcolors;
         switch (n->urgency) {
@@ -393,6 +479,8 @@ void notification_init(struct notification *n)
                 n->colors.fg = g_strdup(defcolors.fg);
         if (!n->colors.bg)
                 n->colors.bg = g_strdup(defcolors.bg);
+        if (!n->colors.highlight)
+                n->colors.highlight = g_strdup(defcolors.highlight);
         if (!n->colors.frame)
                 n->colors.frame = g_strdup(defcolors.frame);
 
@@ -403,9 +491,30 @@ void notification_init(struct notification *n)
         /* Process rules */
         rule_apply_all(n);
 
+        if (g_str_has_prefix(n->summary, "DUNST_COMMAND_")) {
+                char *msg = "DUNST_COMMAND_* has been removed, please switch to dunstctl. See #830 for more details. https://github.com/dunst-project/dunst/pull/830";
+                LOG_W("%s", msg);
+                n->body = string_append(n->body, msg, "\n");
+        }
+
+        /* Icon handling */
+        if (STR_EMPTY(n->iconname))
+                g_clear_pointer(&n->iconname, g_free);
+        if (!n->icon && !n->iconname && n->default_icon_name) {
+                n->iconname = g_strdup(n->default_icon_name);
+        }
+        if (!n->icon && !n->iconname)
+                n->iconname = g_strdup(settings.icons[n->urgency]);
+
+
         /* UPDATE derived fields */
         notification_extract_urls(n);
         notification_format_message(n);
+
+        /* Update timeout: dbus_timeout has priority over timeout */
+        if (n->dbus_timeout >= 0)
+                n->timeout = n->dbus_timeout;
+
 }
 
 static void notification_format_message(struct notification *n)
@@ -563,13 +672,14 @@ void notification_update_text_to_render(struct notification *n)
 
         /* print age */
         gint64 hours, minutes, seconds;
-        gint64 t_delta = time_monotonic_now() - n->timestamp;
+        // Timestamp is floored to the second for display purposes -- see queues.c
+        gint64 t_delta = time_monotonic_now() - (n->timestamp - n->timestamp % S2US(1));
 
         if (settings.show_age_threshold >= 0
             && t_delta >= settings.show_age_threshold) {
-                hours   = t_delta / G_USEC_PER_SEC / 3600;
-                minutes = t_delta / G_USEC_PER_SEC / 60 % 60;
-                seconds = t_delta / G_USEC_PER_SEC % 60;
+                hours   = US2S(t_delta) / 3600;
+                minutes = US2S(t_delta) / 60 % 60;
+                seconds = US2S(t_delta) % 60;
 
                 char *new_buf;
                 if (hours > 0) {
@@ -592,31 +702,51 @@ void notification_update_text_to_render(struct notification *n)
 }
 
 /* see notification.h */
-void notification_do_action(const struct notification *n)
+void notification_do_action(struct notification *n)
 {
+        assert(n->default_action_name);
+        
         if (g_hash_table_size(n->actions)) {
-                if (g_hash_table_contains(n->actions, "default")) {
-                        signal_action_invoked(n, "default");
+                if (g_hash_table_contains(n->actions, n->default_action_name)) {
+                        signal_action_invoked(n, n->default_action_name);
                         return;
                 }
-                if (g_hash_table_size(n->actions) == 1) {
+                if (strcmp(n->default_action_name, "default") == 0 && g_hash_table_size(n->actions) == 1) {
                         GList *keys = g_hash_table_get_keys(n->actions);
                         signal_action_invoked(n, keys->data);
                         g_list_free(keys);
                         return;
                 }
-                context_menu();
+                notification_open_context_menu(n);
 
-        } else if (n->urls) {
-                if (strstr(n->urls, "\n"))
-                        context_menu();
-                else
-                        open_browser(n->urls);
         }
 }
 
+/* see notification.h */
+void notification_open_url(struct notification *n)
+{
+        if (!n->urls) {
+                LOG_W("There are no URL's in this notification");
+                return;
+        }
+        if (strstr(n->urls, "\n"))
+                notification_open_context_menu(n);
+        else
+                open_browser(n->urls);
+}
+
+/* see notification.h */
+void notification_open_context_menu(struct notification *n)
+{
+        GList *notifications = NULL;
+        notifications = g_list_append(notifications, n);
+        notification_lock(n);
+
+        context_menu_for(notifications);
+}
+
 void notification_invalidate_actions(struct notification *n) {
         g_hash_table_remove_all(n->actions);
 }
 
-/* vim: set tabstop=8 shiftwidth=8 expandtab textwidth=0: */
+/* vim: set ft=c tabstop=8 shiftwidth=8 expandtab textwidth=0: */
