@@ -147,6 +147,8 @@ struct settings {
         int progress_bar_min_width;
         int progress_bar_max_width;
         int progress_bar_frame_width;
+        int progress_bar_corner_radius;
+        int icon_corner_radius;
         bool progress_bar;
         enum zwlr_layer_shell_v1_layer layer;
         enum origin_values origin;
