@@ -1064,6 +1064,26 @@ static const struct setting allowed_settings[] = {
                 .parser = NULL,
                 .parser_data = NULL,
         },
+        {
+                .name = "progress_bar_corner_radius",
+                .section = "global",
+                .description = "Progress bar corner radius",
+                .type = TYPE_INT,
+                .default_value = "0",
+                .value = &settings.progress_bar_corner_radius,
+                .parser = NULL,
+                .parser_data = NULL,
+        },
+        {
+                .name = "icon_corner_radius",
+                .section = "global",
+                .description = "Icon corner radius",
+                .type = TYPE_INT,
+                .default_value = "0",
+                .value = &settings.icon_corner_radius,
+                .parser = NULL,
+                .parser_data = NULL,
+        },
         {
                 .name = "progress_bar",
                 .section = "global",
@@ -1445,7 +1465,7 @@ static const struct setting allowed_settings[] = {
                 .section = "global",
                 .description = "Maximum number of notifications allowed",
                 .type = TYPE_INT,
-                .default_value = "0",
+                .default_value = "20",
                 .value = &settings.notification_limit,
                 .parser = NULL,
                 .parser_data = NULL,
