@@ -69,12 +69,12 @@ static double screen_dpi_get_from_xft(void)
         return screen_dpi_xft_cache;
 }
 
-static double screen_dpi_get_from_monitor(struct screen_info *scr)
+static double screen_dpi_get_from_monitor(const struct screen_info *scr)
 {
         return (double)scr->h * 25.4 / (double)scr->mmh;
 }
 
-double screen_dpi_get(struct screen_info *scr)
+double screen_dpi_get(const struct screen_info *scr)
 {
         if (  ! settings.force_xinerama
              && settings.per_monitor_dpi)
@@ -156,6 +156,7 @@ void randr_update(void)
                 screens[i].w = m[i].width;
                 screens[i].h = m[i].height;
                 screens[i].mmh = m[i].mheight;
+                screens[i].dpi = screen_dpi_get(&screens[i]);
         }
 
         XRRFreeMonitors(m);
@@ -242,7 +243,7 @@ bool window_is_fullscreen(Window window)
         ASSERT_OR_RET(window, false);
 
         Atom has_wm_state = XInternAtom(xctx.dpy, "_NET_WM_STATE", True);
-        if (has_wm_state == None){
+        if (has_wm_state == None) {
                 return false;
         }
 
@@ -300,44 +301,48 @@ bool window_is_fullscreen(Window window)
  * Select the screen on which the Window
  * should be displayed.
  */
-struct screen_info *get_active_screen(void)
+const struct screen_info *get_active_screen(void)
 {
         int ret = 0;
         bool force_follow_mouse = false;
-        if (settings.monitor > 0 && settings.monitor < screens_len) {
-                ret = settings.monitor;
-                goto sc_cleanup;
-        }
-
-        x_follow_setup_error_handler();
 
         if (settings.f_mode == FOLLOW_NONE) {
-                ret = XDefaultScreen(xctx.dpy);
+                if (settings.monitor >= 0 && settings.monitor < screens_len) {
+                        ret = settings.monitor;
+                }
                 goto sc_cleanup;
-
         } else {
                 int x, y;
                 assert(settings.f_mode == FOLLOW_MOUSE
                                 || settings.f_mode == FOLLOW_KEYBOARD);
+
+                x_follow_setup_error_handler();
+
                 Window root =
                         RootWindow(xctx.dpy, DefaultScreen(xctx.dpy));
 
                 if (settings.f_mode == FOLLOW_KEYBOARD) {
                         Window focused = get_focused_window();
 
-                        if (focused == 0) {
+                        if (!focused) {
                                 /*
-                                 * This can happen in the case that the user
-                                 * just has the root window open, eg. in empty
-                                 * tags in dwm or similar window managers. In
-                                 * that case, fall back to FOLLOW_MOUSE, since
-                                 * it probably still has the right screen.
+                                 * If no window is focused, or the focus is set
+                                 * to dynamically change to the root window of
+                                 * the screen the pointer is on, force following
+                                 * the mouse.
                                  */
                                 force_follow_mouse = true;
                         } else {
                                 Window child_return;
-                                XTranslateCoordinates(xctx.dpy, focused, root,
-                                                0, 0, &x, &y, &child_return);
+                                /*
+                                 * The window with input focus might be on a
+                                 * different X screen. Use the mouse location
+                                 * in that case.
+                                 */
+                                force_follow_mouse = !XTranslateCoordinates(
+                                                        xctx.dpy, focused,root,
+                                                        0, 0, &x, &y,
+                                                        &child_return);
                         }
                 }
 
@@ -368,7 +373,7 @@ struct screen_info *get_active_screen(void)
                         goto sc_cleanup;
 
                 /* something seems to be wrong. Fall back to default */
-                ret = XDefaultScreen(xctx.dpy);
+                ret = 0;
                 goto sc_cleanup;
         }
 sc_cleanup:
@@ -384,32 +389,13 @@ sc_cleanup:
  */
 static Window get_focused_window(void)
 {
-        Window focused = 0;
-        Atom type;
-        int format;
-        unsigned long nitems, bytes_after;
-        unsigned char *prop_return = NULL;
-        Window root = RootWindow(xctx.dpy, DefaultScreen(xctx.dpy));
-        Atom netactivewindow =
-            XInternAtom(xctx.dpy, "_NET_ACTIVE_WINDOW", false);
+        Window focused, root = RootWindow(xctx.dpy, DefaultScreen(xctx.dpy));
+        int ignored;
 
-        XGetWindowProperty(xctx.dpy,
-                           root,
-                           netactivewindow,
-                           0L,
-                           sizeof(Window),
-                           false,
-                           XA_WINDOW,
-                           &type,
-                           &format,
-                           &nitems,
-                           &bytes_after,
-                           &prop_return);
-        if (prop_return) {
-                focused = *(Window *)prop_return;
-                XFree(prop_return);
-        }
+        XGetInputFocus(xctx.dpy, &focused, &ignored);
 
+        if (focused == None || focused == PointerRoot || focused == root)
+                focused = 0;
         return focused;
 }
 
@@ -438,4 +424,4 @@ static int FollowXErrorHandler(Display *display, XErrorEvent *e)
 
         return 0;
 }
-/* vim: set tabstop=8 shiftwidth=8 expandtab textwidth=0: */
+/* vim: set ft=c tabstop=8 shiftwidth=8 expandtab textwidth=0: */
