@@ -4,6 +4,7 @@
 
 #include "../src/option_parser.h"
 #include "../src/settings.h"
+#include "../src/icon.h"
 
 extern const char *base;
 
@@ -47,9 +48,9 @@ TEST test_notification_is_duplicate(void)
         ASSERTm("One of the notifications got corrupted during test",
                 notification_is_duplicate(a, b));
 
-        enum icon_position icon_setting_tmp = settings.icon_position;
+        enum icon_position icon_setting_tmp = a->icon_position;
 
-        settings.icon_position = ICON_OFF;
+        a->icon_position = ICON_OFF;
         ASSERT(notification_is_duplicate(a, b));
         //Setting pointer to a random value since we are checking for null
         char *icon_id = b->icon_id;
@@ -58,13 +59,15 @@ TEST test_notification_is_duplicate(void)
                 notification_is_duplicate(a, b));
         b->icon_id = icon_id;
 
-        settings.icon_position = ICON_LEFT;
+        a->icon_position = ICON_LEFT;
+        b->icon_position = ICON_LEFT;
         CHECK_CALL(test_notification_is_duplicate_field(&(b->icon_id), a, b));
 
-        settings.icon_position = ICON_RIGHT;
+        a->icon_position = ICON_RIGHT;
+        b->icon_position = ICON_RIGHT;
         CHECK_CALL(test_notification_is_duplicate_field(&(b->icon_id), a, b));
 
-        settings.icon_position = icon_setting_tmp;
+        a->icon_position = icon_setting_tmp;
 
         ASSERT(notification_is_duplicate(a, b));
 
@@ -134,11 +137,9 @@ static struct notification *notification_load_icon_with_scaling(int min_icon_siz
 
         GVariant *rawIcon = notification_setup_raw_image(path);
 
-        settings.min_icon_size = min_icon_size;
-        settings.max_icon_size = max_icon_size;
+        n->min_icon_size = min_icon_size;
+        n->max_icon_size = max_icon_size;
         notification_icon_replace_data(n, rawIcon);
-        settings.min_icon_size = 0;
-        settings.max_icon_size = 0;
 
         g_variant_unref(rawIcon);
         g_free(path);
@@ -150,8 +151,8 @@ TEST test_notification_icon_scaling_toosmall(void)
 {
         struct notification *n = notification_load_icon_with_scaling(20, 100);
 
-        ASSERT_EQ(gdk_pixbuf_get_width(n->icon), 20);
-        ASSERT_EQ(gdk_pixbuf_get_height(n->icon), 20);
+        ASSERT_EQ(get_icon_width(n->icon, 1), 20);
+        ASSERT_EQ(get_icon_height(n->icon, 1), 20);
 
         notification_unref(n);
 
@@ -163,8 +164,8 @@ TEST test_notification_icon_scaling_toolarge(void)
 {
         struct notification *n = notification_load_icon_with_scaling(5, 10);
 
-        ASSERT_EQ(gdk_pixbuf_get_width(n->icon), 10);
-        ASSERT_EQ(gdk_pixbuf_get_height(n->icon), 10);
+        ASSERT_EQ(get_icon_width(n->icon, 1), 10);
+        ASSERT_EQ(get_icon_height(n->icon, 1), 10);
 
         notification_unref(n);
 
@@ -175,8 +176,8 @@ TEST test_notification_icon_scaling_notconfigured(void)
 {
         struct notification *n = notification_load_icon_with_scaling(0, 0);
 
-        ASSERT_EQ(gdk_pixbuf_get_width(n->icon), 16);
-        ASSERT_EQ(gdk_pixbuf_get_height(n->icon), 16);
+        ASSERT_EQ(get_icon_width(n->icon, 1), 16);
+        ASSERT_EQ(get_icon_height(n->icon, 1), 16);
 
         notification_unref(n);
 
@@ -187,8 +188,8 @@ TEST test_notification_icon_scaling_notneeded(void)
 {
         struct notification *n = notification_load_icon_with_scaling(10, 20);
 
-        ASSERT_EQ(gdk_pixbuf_get_width(n->icon), 16);
-        ASSERT_EQ(gdk_pixbuf_get_height(n->icon), 16);
+        ASSERT_EQ(get_icon_width(n->icon, 1), 16);
+        ASSERT_EQ(get_icon_height(n->icon, 1), 16);
 
         notification_unref(n);
 
@@ -232,8 +233,6 @@ TEST test_notification_maxlength(void)
 SUITE(suite_notification)
 {
         cmdline_load(0, NULL);
-        char *config_path = g_strconcat(base, "/data/dunstrc.default", NULL);
-        load_settings(config_path);
 
         RUN_TEST(test_notification_is_duplicate);
         RUN_TEST(test_notification_replace_single_field);
@@ -275,9 +274,6 @@ SUITE(suite_notification)
         g_clear_pointer(&a, notification_unref);
 
         RUN_TEST(test_notification_maxlength);
-
-        g_clear_pointer(&settings.icon_path, g_free);
-        g_free(config_path);
 }
 
 /* vim: set tabstop=8 shiftwidth=8 expandtab textwidth=0: */
