@@ -301,6 +301,24 @@ TEST test_queue_notification_skip_display_redisplayed_by_random_id(void) {
         PASS();
 }
 
+TEST test_queue_history_clear(void)
+{
+        struct notification *n;
+
+        n = test_notification("n", -1);
+
+        queues_init();
+        queues_notification_insert(n);
+        queues_history_push_all();
+
+        QUEUE_LEN_ALL(0, 0, 1);
+        queues_history_clear();
+        QUEUE_LEN_ALL(0, 0, 0);
+        queues_teardown();
+
+        PASS();
+}
+
 TEST test_queue_history_overfull(void)
 {
         settings.history_length = 10;
@@ -363,6 +381,42 @@ TEST test_queue_history_pushall(void)
         PASS();
 }
 
+TEST test_queue_history_remove_by_id(void)
+{
+        settings.history_length = 5;
+        settings.indicate_hidden = false;
+        settings.notification_limit = 0;
+
+        queues_init();
+
+        struct notification *n;
+        struct notification *n1 = NULL;
+
+
+        for (int i = 0; i < 5; i++) {
+                char name[] = { 'n', '0'+i, '\0' }; // n<i>
+                n = test_notification(name, -1);
+                queues_notification_insert(n);
+
+                // Store notification at arbitrary position to remove
+                if(i==1) {
+                        n1 = n;
+                }
+        }
+        queues_history_push_all();
+
+        queues_history_remove_by_id(n->id);
+        queues_history_remove_by_id(n1->id);
+
+        QUEUE_LEN_ALL(0, 0, 3);   
+        
+        QUEUE_NOT_CONTAINS(HIST, n);
+        QUEUE_NOT_CONTAINS(HIST, n1);
+
+        queues_teardown();
+        PASS();
+}
+
 TEST test_queue_init(void)
 {
         queues_init();
@@ -420,25 +474,69 @@ TEST test_datachange_endless(void)
 }
 
 TEST test_datachange_endless_agethreshold(void)
+{
+        const gint64 AGE_THRESHOLD = S2US(5);
+        settings.show_age_threshold = AGE_THRESHOLD;
+
+        gint64 cur_time = 0;
+        queues_init();
+
+        struct notification *n = test_notification("n", 0);
+        n->timestamp = cur_time;
+
+        queues_notification_insert(n);
+        queues_update(STATUS_NORMAL, cur_time);
+
+        ASSERT_IN_RANGEm("Age threshold is activated and the next wakeup should be less than a second away",
+                AGE_THRESHOLD - S2US(1)/2, queues_get_next_datachange(cur_time + AGE_THRESHOLD - S2US(1)), S2US(1)/2);
+
+        ASSERT_IN_RANGEm("Age threshold is activated and the next wakeup should be less than the age threshold",
+                AGE_THRESHOLD / 2, queues_get_next_datachange(cur_time), AGE_THRESHOLD/2);
+
+        settings.show_age_threshold = S2US(0);
+        ASSERT_IN_RANGEm("Age threshold is activated and the next wakeup should be less than a second away",
+                S2US(1)/2, queues_get_next_datachange(cur_time), S2US(1)/2);
+
+        queues_teardown();
+        PASS();
+}
+
+TEST test_datachange_agethreshold_at_second(void)
 {
         settings.show_age_threshold = S2US(5);
 
         queues_init();
 
-        struct notification *n = test_notification("n", 0);
+        const gint64 NOTIF_DELTA = 200;
 
+        gint64 cur_time = 500*1000; // T = 0.5s
+        struct notification *n = test_notification("n0", 0);
+        n->timestamp = cur_time;
         queues_notification_insert(n);
-        queues_update(STATUS_NORMAL, time_monotonic_now());
 
-        ASSERT_IN_RANGEm("Age threshold is activated and the next wakeup should be less than a second away",
-                S2US(1)/2, queues_get_next_datachange(time_monotonic_now() + S2US(4)), S2US(1)/2);
+        cur_time += NOTIF_DELTA;
+        n = test_notification("n1", 0);
+        n->timestamp = cur_time;
+        queues_notification_insert(n);
 
-        ASSERT_IN_RANGEm("Age threshold is activated and the next wakeup should be less than the age threshold",
-                settings.show_age_threshold/2, queues_get_next_datachange(time_monotonic_now()), settings.show_age_threshold/2);
+        queues_update(STATUS_NORMAL, cur_time);
 
-        settings.show_age_threshold = S2US(0);
-        ASSERT_IN_RANGEm("Age threshold is activated and the next wakeup should be less than a second away",
-                S2US(1)/2, queues_get_next_datachange(time_monotonic_now()), S2US(1)/2);
+        // The next update should be when age must be first displayed, at T=5.5s
+        ASSERTm("Age threshold is activated, first wakeup should be at 5.5s",
+                        queues_get_next_datachange(cur_time) == S2US(5) + 500000);
+
+        const int NB_MICROSECS = 6;
+        const gint64 MICROSECS[] = {
+                0, 10, NOTIF_DELTA, NOTIF_DELTA + 1, 124131, S2US(1)-1
+        };
+        for(gint64 base_time = S2US(5); base_time <= S2US(7); base_time += S2US(1)) {
+                for(int musec_id = 0; musec_id < NB_MICROSECS; ++musec_id) {
+                        cur_time = base_time + MICROSECS[musec_id];
+                        gint64 next_wakeup = queues_get_next_datachange(cur_time);
+                        ASSERTm("Age threshold is activated, next wakeup should be at next turn of second",
+                                        next_wakeup == base_time + S2US(1));
+                }
+        }
 
         queues_teardown();
         PASS();
@@ -448,19 +546,21 @@ TEST test_datachange_queues(void)
 {
         queues_init();
 
+        gint64 cur_time = 0;
         struct notification *n = test_notification("n", 10);
+        n->timestamp = cur_time;
 
         queues_notification_insert(n);
         ASSERTm("The inserted notification is inside the waiting queue, so it should get ignored.",
-               queues_get_next_datachange(time_monotonic_now()) < S2US(0));
+               queues_get_next_datachange(cur_time) < 0);
 
         queues_update(STATUS_NORMAL, time_monotonic_now());
         ASSERT_IN_RANGEm("The notification has to get closed in less than its timeout",
-               S2US(10)/2, queues_get_next_datachange(time_monotonic_now()), S2US(10)/2);
+               S2US(10)/2, queues_get_next_datachange(cur_time), S2US(10)/2);
 
         queues_notification_close(n, REASON_UNDEF);
         ASSERTm("The inserted notification is inside the history queue, so it should get ignored",
-               queues_get_next_datachange(time_monotonic_now()) < S2US(0));
+               queues_get_next_datachange(cur_time) < 0);
 
         queues_teardown();
         PASS();
@@ -470,23 +570,29 @@ TEST test_datachange_ttl(void)
 {
         struct notification *n;
         queues_init();
+        gint64 cur_time = 0;
 
         n = test_notification("n1", 15);
+        n->timestamp = cur_time;
 
         queues_notification_insert(n);
-        queues_update(STATUS_NORMAL, time_monotonic_now());
+        queues_update(STATUS_NORMAL, cur_time);
         ASSERT_IN_RANGEm("The notification has to get closed in less than its timeout.",
-               n->timeout/2, queues_get_next_datachange(time_monotonic_now()), n->timeout/2);
+               n->timeout/2, queues_get_next_datachange(cur_time), n->timeout/2);
 
+        cur_time += 500; // microseconds
         n = test_notification("n2", 10);
+        n->timestamp = cur_time;
 
         queues_notification_insert(n);
-        queues_update(STATUS_NORMAL, time_monotonic_now());
+        queues_update(STATUS_NORMAL, cur_time);
         ASSERT_IN_RANGEm("The timeout of the second notification has to get used as sleep time now.",
-               n->timeout/2, queues_get_next_datachange(time_monotonic_now()), n->timeout/2);
+               cur_time + n->timeout/2, queues_get_next_datachange(cur_time), n->timeout/2);
 
-        ASSERT_EQm("The notification already timed out. You have to answer with 0.",
-               S2US(0), queues_get_next_datachange(time_monotonic_now() + S2US(10)));
+        cur_time += S2US(10);
+
+        ASSERT_EQm("The notification already timed out. You have to answer with the current time.",
+               cur_time, queues_get_next_datachange(cur_time));
 
         queues_teardown();
         PASS();
@@ -987,10 +1093,13 @@ SUITE(suite_queues)
         RUN_TEST(test_datachange_beginning_empty);
         RUN_TEST(test_datachange_endless);
         RUN_TEST(test_datachange_endless_agethreshold);
+        RUN_TEST(test_datachange_agethreshold_at_second);
         RUN_TEST(test_datachange_queues);
         RUN_TEST(test_datachange_ttl);
+        RUN_TEST(test_queue_history_clear);
         RUN_TEST(test_queue_history_overfull);
         RUN_TEST(test_queue_history_pushall);
+        RUN_TEST(test_queue_history_remove_by_id);
         RUN_TEST(test_queue_init);
         RUN_TEST(test_queue_insert_id_invalid);
         RUN_TEST(test_queue_insert_id_replacement);
